/*-------------------------------------------------------------------------------*/
#include <stdio.h>
#include <math.h>
#include <mpi.h>
#include "function_declartion.h"
#include "constant.h"
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  Eigen_Value( TEMP *TP, double *par, double *BC, double *x, double *REV, double *IEV )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int NV;
  NV  =  TP->NV;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j;
  double **a;
  double *t_a, *t_REV, *t_IEV;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  a      =  malloc_2d( NV, NV );
  t_a    =  MALLOC_1D( NV*NV  );
  t_REV  =  MALLOC_1D( NV     );
  t_IEV  =  MALLOC_1D( NV     );
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  M_EV( *TP, par, BC, x, a );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 0; i <= NV-1; i++ )
    for( j = 0; j <= NV-1; j++ )
      t_a[NV*i+j]  =  a[i+1][j+1];
  clcegn_( &NV, &NV, t_a, t_REV, t_IEV );
  for( i = 0; i <= NV-1; i++ ){
    REV[i+1]  =  t_REV[i];
    IEV[i+1]  =  t_IEV[i];
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_2d( a     );
  FREE_1D( t_a   );
  FREE_1D( t_REV );
  FREE_1D( t_IEV );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  N_UnstableEigenValue( TP, par, REV, IEV );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  Eigen_Vector( TEMP *TP, double *par, double *BC, double *d_x, double *x, double *REV, double *IEV )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    N, NV, NA, np;
  double dx;
  N   =  TP->N;
  NV  =  TP->NV;
  NA  =  TP->NA;
  np  =  TP->np;
  dx  =  TP->dx;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j;
  int    N_EVC;
  double JUDG_EVC;
  double minEV;
  int    *i_E;
  double **a;
  double **EVC;
  double **JREVC, **JIEVC;
  int    *t_i_E;
  double *t_a, *t_REV, *t_IEV, *t_EVC;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  i_E    =  malloc_1i( NV     );
  a      =  malloc_2d( NV, NV );
  EVC    =  malloc_2d( NV, NV );
  t_i_E  =  MALLOC_1I( NV     );
  t_a    =  MALLOC_1D( NV*NV  );
  t_REV  =  MALLOC_1D( NV     );
  t_IEV  =  MALLOC_1D( NV     );
  t_EVC  =  MALLOC_1D( NV*NV  );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  if( np == 0 )  JUDG_EVC = (double)JUDG_CALCULATE_EVC_S;
  else           JUDG_EVC = (double)JUDG_CALCULATE_EVC_P;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  M_EV( *TP, par, BC, x, a );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*if(TP->RANK==0){*/
/*FILE*fp;*/
/*fp=fopen_mode("112",'w');*/
/*for(i=1;i<=NV;i++)*/
/*for(j=1;j<=NV;j++)*/
/*fprintf(fp,"%20.15lf\n",a[i][j]);*/
/*fclose(fp);*/
/*}*/
/*  MPI_Barrier( MPI_COMM_WORLD );*/
/*  MPI_Finalize();*/
/*exit(1);*/
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*if(TP->RANK==0){*/
/*FILE *fp;*/
/*fp=fopen_mode("112",'r');*/
/*  for( i = 1; i <= NV; i++ )*/
/*    for( j = 1; j <= NV; j++ )*/
/*      fscanf(fp,"%lf",&a[i][j]);*/
/*fclose(fp);*/
/*}*/
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 0; i <= NV-1; i++ )
    for( j = 0; j <= NV-1; j++ )
      t_a[NV*j+i]  =  a[i+1][j+1];
  int    *iv1;
  double *fm1, *fv1, *fv2, *fv3;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  iv1 = MALLOC_1I( NV    );
  fm1 = MALLOC_1D( NV*NV );
  fv1 = MALLOC_1D( NV    );
  fv2 = MALLOC_1D( NV    );
  fv3 = MALLOC_1D( NV    );
  clcegnv_( &NV, &NV, &np, iv1, fm1, fv1, fv2, fv3, t_a, t_REV, t_IEV, t_EVC, t_i_E, &N_EVC, &JUDG_EVC );
  FREE_1I( iv1 );
  FREE_1D( fm1 );
  FREE_1D( fv1 );
  FREE_1D( fv2 );
  FREE_1D( fv3 );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 0; i <= NV-1; i++ ){
    REV[i+1]  =  t_REV[i];
    IEV[i+1]  =  t_IEV[i];
  }
  if( N_EVC == 0 )   goto END;
  for( i = 0; i <= NV-1; i++ ){
    i_E[i+1]  =  t_i_E[i];
    for( j = 0; j <= NV-1; j++ )
      EVC[i+1][j+1]  =  t_EVC[NV*j+i];
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  if( TP->np == 0 ){   minEV = fabs(REV[i_E[1]]);
                       for( i = 2; i <= N_EVC; i++ )   minEV  =  MIN( minEV, fabs(REV[i_E[i]]) );   }
  else{                minEV = sqrt(REV[i_E[1]]*REV[i_E[1]]+IEV[i_E[1]]*IEV[i_E[1]]);
                       for( i = 2; i <= N_EVC; i++ )   minEV  =  MAX( minEV, sqrt(REV[i_E[i]]*REV[i_E[i]]+IEV[i_E[i]]*IEV[i_E[i]]) );   }
/*  else{                minEV = fabs(sqrt(REV[i_E[1]]*REV[i_E[1]]+IEV[i_E[1]]*IEV[i_E[1]])-1.0);*/
/*                       for( i = 2; i <= N_EVC; i++ )   minEV  =  MIN( minEV, fabs(sqrt(REV[i_E[i]]*REV[i_E[i]]+IEV[i_E[i]]*IEV[i_E[i]])-1.0) );   }*/
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  JREVC  =  malloc_2d( NV+NA, N_EVC );
  JIEVC  =  malloc_2d( NV+NA, N_EVC );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 1; i <= N_EVC; i++ ){
    if( 0.0  ==  IEV[i_E[i]] ){
      for( j = 1; j <= NV; j++ ){
        JREVC[j][i]  =  EVC[j][i];
        JIEVC[j][i]  =  0.0;
      }
    }else{
      for( j = 1; j <= NV; j++ ){
        JREVC[j][i  ]  =  EVC[j][i  ];
        JIEVC[j][i  ]  =  EVC[j][i+1];
        JREVC[j][i+1]  =     JREVC[j][i];
        JIEVC[j][i+1]  =  -  JIEVC[j][i];
      }
      i ++;
    }
  }
  for( i = 1; i <= N_EVC; i++ ){
    for( j = NV+1; j <= NV+NA; j++ ){
      JREVC[j][i]  =  0.0;
      JIEVC[j][i]  =  0.0;
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 1; i <= N_EVC; i++ ){
    TP->SUM_RIEVC  =  0.0;
    for( j = 1; j <= N-1; j++ ){
      TP->SUM_RIEVC  +=  0.5 * dx * ( JREVC[j  ][i]*JREVC[j  ][i]+JREVC[j+1  ][i]*JREVC[j+1  ][i] );
      TP->SUM_RIEVC  +=  0.5 * dx * ( JREVC[j+N][i]*JREVC[j+N][i]+JREVC[j+1+N][i]*JREVC[j+1+N][i] );
      TP->SUM_RIEVC  +=  0.5 * dx * ( JIEVC[j  ][i]*JIEVC[j  ][i]+JIEVC[j+1  ][i]*JIEVC[j+1  ][i] );
      TP->SUM_RIEVC  +=  0.5 * dx * ( JIEVC[j+N][i]*JIEVC[j+N][i]+JIEVC[j+1+N][i]*JIEVC[j+1+N][i] );
    }
    TP->SUM_RIEVC  =  sqrt( TP->SUM_RIEVC );
    for( j = 1; j <= NV; j++ ){
      JREVC[j][i]  /=  TP->SUM_RIEVC;
      JIEVC[j][i]  /=  TP->SUM_RIEVC;
    }
    if( TP->RANK  ==  0 )
      output2( i, N_EVC, minEV, TP, par, BC, d_x, x, i_E, REV, IEV, JREVC, JIEVC, 'w' );
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_2d( JREVC  );
  free_2d( JIEVC  );
  END:
  free_1i( i_E    );
  free_2d( a      );
  free_2d( EVC    );
  FREE_1I( t_i_E  );
  FREE_1D( t_a    );
  FREE_1D( t_REV  );
  FREE_1D( t_IEV  );
  FREE_1D( t_EVC  );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  N_UnstableEigenValue( TP, par, REV, IEV );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/
