/*-------------------------------------------------------------------------------*/
#include <stdio.h>
#include <mpi.h>
#include "StructType_Definition.h"
#include "function_declartion.h"
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct_11( int L, double *A, double *B, double *C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i;
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
  *C  =  0.0;
  for( i = 1; i <= L; i++ )
    (*C)  +=  A[i] * B[i];
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct( int L, int M, double **A, int N, double **B, double **C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k;
  double tmp;
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
  for( i = 1; i <= L; i++ ){
    for( j = 1; j <= N; j++ ){
      tmp  =  0.0;
      for( k = 1; k <= M; k++ )
        tmp  +=  A[i][k] * B[k][j];
      C[i][j]  =  tmp;
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct_22s( int L, int M, double **A, int N, double **B, double **C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k;
  double tmp;
  double **D;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  D  =  malloc_2d( L, N );
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
  for( i = 1; i <= L; i++ ){
    for( j = 1; j <= N; j++ ){
      tmp  =  0.0;
      for( k = 1; k <= M; k++ )
        tmp  +=  A[i][k] * B[k][j];
      D[i][j]  =  tmp;
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 1; i <= L; i++ )
    for( j = 1; j <= N; j++ )
       C[i][j]  =  D[i][j];
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_2d( D );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct_21( int L, int M, double **A, double *B, double *C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k;
  double tmp;
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
  for( i = 1; i <= L; i++ ){
    tmp  =  0.0;
    for( j = 1; j <= M; j++ )
      tmp  +=  A[i][j] * B[j];
    C[i]  =  tmp;
  }
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct_21s( int L, int M, double **A, double *B, double *C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k;
  double tmp;
  double *D;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  D  =  malloc_1d( L );
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
  for( i = 1; i <= L; i++ ){
    tmp  =  0.0;
    for( j = 1; j <= M; j++ )
      tmp  +=  A[i][j] * B[j];
    D[i]  =  tmp;
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 1; i <= L; i++ )
    C[i]  =  D[i];
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_1d( D );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct_MPI( int PROCS, int RANK, int L, int M, double **A, int N, double **B, double **C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  void  Partition_1dim( int PROCS, int RANK, int Ni, int Nj, int *start, int *end, int *len, int *disp );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k;
  double tmp;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    is, ie;
  int    *len, *disp;
  double **t_C;
  len   =  malloc_1i( PROCS+1 );
  disp  =  malloc_1i( PROCS+1 );
  Partition_1dim( PROCS, RANK, L, N, &is, &ie, len, disp );
  t_C   =  malloc_2d( ie-is+1, L );
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
  for( i = is; i <= ie; i++ ){
    for( j = 1; j <= N; j++ ){
      tmp  =  0.0;
      for( k = 1; k <= M; k++ )
        tmp  +=  A[i][k] * B[k][j];
      t_C[i-is+1][j]  =  tmp;
    }
  }
  MPI_Allgatherv( &t_C[1][1], len[RANK+1], MPI_DOUBLE, &C[1][1], &len[1], &disp[1], MPI_DOUBLE, MPI_COMM_WORLD );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_1i( len  );
  free_1i( disp );
  free_2d( t_C  );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct_MPI1( int PROCS, int RANK, int is    , int ie, int *len  , int *disp , double **t_C, \
                          int L    , int M   , double **A, int N , double **B, double **C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k;
  double tmp;
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
  for( i = is; i <= ie; i++ ){
    for( j = 1; j <= N; j++ ){
      tmp  =  0.0;
      for( k = 1; k <= M; k++ )
        tmp  +=  A[i][k] * B[k][j];
      t_C[i-is+1][j]  =  tmp;
    }
  }
  MPI_Allgatherv( &t_C[1][1], len[RANK+1], MPI_DOUBLE, &C[1][1], &len[1], &disp[1], MPI_DOUBLE, MPI_COMM_WORLD );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct_22s_MPI( int PROCS, int RANK, int L, int M, double **A, int N, double **B, double **C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  void  Partition_1dim( int PROCS, int RANK, int Ni, int Nj, int *start, int *end, int *len, int *disp );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k;
  double tmp;
  double **D;
  D  =  malloc_2d( L, N );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    is, ie;
  int    *len, *disp;
  double **t_C;
  len   =  malloc_1i( PROCS+1 );
  disp  =  malloc_1i( PROCS+1 );
  Partition_1dim( PROCS, RANK, L, N, &is, &ie, len, disp );
  t_C   =  malloc_2d( ie-is+1, L );
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
  for( i = is; i <= ie; i++ ){
    for( j = 1; j <= N; j++ ){
      tmp  =  0.0;
      for( k = 1; k <= M; k++ )
        tmp  +=  A[i][k] * B[k][j];
      t_C[i-is+1][j]  =  tmp;
    }
  }
  MPI_Allgatherv( &t_C[1][1], len[RANK+1], MPI_DOUBLE, &D[1][1], &len[1], &disp[1], MPI_DOUBLE, MPI_COMM_WORLD );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 1; i <= L; i++ )
    for( j = 1; j <= N; j++ )
      C[i][j]  =  D[i][j];
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_2d( D    );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_1i( len  );
  free_1i( disp );
  free_2d( t_C  );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/
