/*-------------------------------------------------------------------------------*/
#include <stdio.h>
#include <mpi.h>
#include <math.h>
#include "function_declartion.h"
#include "read_write_file.h"
#include "constant.h"
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  Newton_Method( TEMP *TP, int *c_G, double *par, double *BC, double *t__d_x, double *t__p_x, double *t__x )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int N_VAR, N, NM, NV, np, nv, nk, npf, na;
  N_VAR  =  TP->N_VAR;
  N      =  TP->N;
  NM     =  TP->NM;
  NV     =  TP->NV;
  np     =  TP->np;
  nv     =  TP->nv;
  nk     =  TP->nk;
  npf    =  TP->npf;
  na     =  TP->na;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, c_EES, iteration, DEL_ROW_A, DEL_COL_A, MAX_A;
  int    *IP;
  double CC_Norm, BEST_CC_Norm=0.0;
  double *d_x, *p_x, *x, *F, *RS, *tRS;
  double **PDF, **keep_p_x=NULL, **a, **M;
  char   *PRINTF;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  if     (   TP->EES == EES_D    ||  TP->EES == EES_D_M     )    c_EES = EES_D;
  else if(   TP->EES == EES_S_E  ||  TP->EES == EES_S_E_M   )    c_EES = EES_S_E;
  else if(   TP->EES == EES_S_R  ||  TP->EES == EES_S_R_M   )    c_EES = EES_S_R;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  IP        =  malloc_1i( NM*NV+na                 );
  d_x       =  malloc_1d( NM*NV+na                 );
  p_x       =  malloc_1d( NM*NV+na                 );
  x         =  malloc_1d( NM*NV+na                 );
  if( np  ==  0 )
  F         =  malloc_1d(    NV                    );
  RS        =  malloc_1d( NM*NV+na                 );
  tRS       =  malloc_1d( NM*NV+na                 );
  if( np  ==  0 )
  PDF       =  malloc_2d( N_VAR   , NV+N           );
  if(   np == 1  &&  (TP->EES==EES_D_M || TP->EES==EES_S_E_M || TP->EES==EES_S_R_M)   ){
  int N_t  =  (int)(1.0/TP->dt);
  if( 1.0/TP->dt-N_t >= 0.5 )   N_t ++;
  keep_p_x  =  malloc_2d( N_t+1         , NV       );
  }
  int c_CM = 1;
  if(np==0)c_CM=-1;
  if( c_CM  !=  1 )
  a         =  malloc_2d( NM*NV+na      , NM*NV+na );
  else
  a         =  malloc_2d(    NV+na      , NV+na    );
  if( c_CM  ==  1 )
  M         =  malloc_2d( NM*NV, NV+na+1 );
  PRINTF    =  MALLOC_1C( SIZE                     );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  if( c_CM  !=  1 ){
    DEL_ROW_A  =  NM*NV+1;
    DEL_COL_A  =  NM*NV+1;
    MAX_A      =  NM*NV+na;
  }else{
    DEL_ROW_A  =  NV+1;
    DEL_COL_A  =  NV+1;
    MAX_A      =  NV+na;
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  if( npf  ==  1 ){
    TP->ds           =   0.0;
    t__x[NM*NV+np]  +=   TP->ds_PF;
  }
  if( nk  ==  0 ){
    par[TP->CP]   +=  TP->ds;
    t__x[NM*NV+3]  =  par[TP->CP];
  }
  Diffusion( *TP, par );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 1; i <= NM*NV; i++ ){
    d_x[i]  =  t__d_x[i];
    p_x[i]  =  t__p_x[i];
    x  [i]  =  t__x  [i];
  }
  if( np  ==  1 ){
    d_x[NM*NV+np]  =  t__d_x[NM*NV+1];
    p_x[NM*NV+np]  =  t__p_x[NM*NV+1];
    x  [NM*NV+np]  =  t__x  [NM*NV+1];
  }
  if( nv  ==  1 ){
    d_x[NM*NV+np+nv]  =  t__d_x[NM*NV+2];
    p_x[NM*NV+np+nv]  =  t__p_x[NM*NV+2];
    x  [NM*NV+np+nv]  =  t__x  [NM*NV+2];
  }
  if( nk  ==  1 ){
    d_x[NM*NV+na]  =  t__d_x[NM*NV+3];
    p_x[NM*NV+na]  =  t__p_x[NM*NV+3];
    x  [NM*NV+na]  =  t__x  [NM*NV+3];
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*  TP->WX=1.0;*/
/*  TP->WT=1.0;*/
/*  TP->WV=1.0;*/
/*  TP->WA=1.0;*/
/*  if( nk  ==  1 ){*/
/*    for( i = 1; i <= NM*NV; i++ )   x[i          ] += TP->WX*TP->ds*d_x[i          ];*/
/*    if( np == 1 && npf != 1 )       x[NM*NV+np   ] += TP->WT*TP->ds*d_x[NM*NV+np   ];*/
/*    if( nv == 1 )                   x[NM*NV+np+nv] += TP->WV*TP->ds*d_x[NM*NV+np+nv];*/
/*                                    x[NM*NV+na   ] += TP->WA*TP->ds*d_x[NM*NV+na   ];*/
/*    par[TP->CP]=x[NM*NV+na];*/
/*    Diffusion( *TP, par );*/
/*  }*/
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
  iteration  =  0;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  while( 1 ){
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*    if( np  !=  0 ){*/
/*      TP->dt  =  TP->pdt/x[NM*NV+1];*/
/*      if( TP->RANK == 0 )   printf("dt=%.15lf\n",TP->dt);*/
/*    }*/
    if( TP->RANK == 0 )   NW_Gnuplot( iteration, *TP, c_G, par, BC, x );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    if( np  ==  0 ){
      Reaction( *TP, par, x, F );
        if     (   c_EES == EES_D                           )   RS_S    ( *TP, par, BC, x  , F, RS );
        else if(   c_EES == EES_S_E  ||  c_EES == EES_S_R   )   RS_S_SFM( *TP, par,     x  , F, RS );
      if( nv == 1 )
        if     (   c_EES == EES_D                           )   RS_V    ( *TP, par, BC, p_x, x, RS );
        else if(   c_EES == EES_S_E  ||  c_EES == EES_S_R   )   RS_V_SFM( *TP, par,     p_x, x, RS );
      if( nk == 1 )
        if     (   c_EES == EES_D                           )   RS_K    ( *TP, d_x,     p_x, x, RS );
        else if(   c_EES == EES_S_E  ||  c_EES == EES_S_R   )   RS_K_SFM( *TP, d_x,     p_x, x, RS );
      TP->v = x[NV+np+nv];
      NM_Norm( iteration, TP, par, RS, PRINTF, &CC_Norm );
      if(  TP->RANK == 0  )   NM_Stdout( iteration, *TP, CC_Norm, PRINTF, par, x ); 
    }else{
      double **p_aa, **aa, **keep_EEx=NULL;
      if(  c_EES == EES_D  ){
        p_aa =  malloc_2d( 5, NV );
        aa   =  malloc_2d( 5, NV );
      }
      keep_EEx =  malloc_2d( NM, NV+na );
      RS_P( c_EES, TP, par, BC, d_x, p_x, x, RS, keep_p_x, keep_EEx, p_aa, aa );
      if(   TP->RANK == 0   &&   ( iteration == 0  ||  BEST_CC_Norm >= CC_Norm )   ){
         Output_MidleData( *TP, par, BC, d_x, x, WRITE_MIDLEDATA_ITER, 'w' );
         BEST_CC_Norm = CC_Norm;
      }
      NM_Norm( iteration, TP, par, RS, PRINTF, &CC_Norm );
      if(  TP->RANK == 0  )
        NM_Stdout( iteration, *TP, CC_Norm, PRINTF, par, x ); 
      if(  TP->CC == 0  ){
        if(  iteration == 0  ||  TP->PN < CC_Norm  ){
          if  ( c_CM  !=  1 ){   M_PF( c_EES, TP, par, BC, d_x, p_x, x, RS, keep_p_x, keep_EEx, p_aa, aa, a    );             }
          else{                  M_CM( c_EES, TP, par, BC, d_x, p_x, x, RS, keep_p_x, keep_EEx, p_aa, aa, a, M );   NM = 1;   }
        }else if( c_CM  ==  1 ){
          int    i_NM, j;
          double *C;
          double **MM;
          C   =  malloc_1d( NV     );
          MM  =  malloc_2d( NV, NV );
          for( i_NM = 1; i_NM <= NM; i_NM++ )
            for( i = 1; i <= NV; i++ )
              M[(i_NM-1)*NV+i][NV+na+1]  =  RS[(i_NM-1)*NV+i];
          for( i_NM = 2; i_NM <= NM; i_NM++ ){
            for( i = 1; i <= NV; i++ )
              for( j = 1; j <= NV; j++ )
                MM[i][j]  =  M[(i_NM-1)*NV+i][j];
            MatrixProduct_21( NV, NV, MM, RS, C );
            for( i = 1; i <= NV; i++ )
              RS[i]  =  C[i] + RS[(i_NM-1)*NV+i];
          }
          free_1d( C  );
          free_2d( MM );
          for( i = 1; i <= na; i++ )
            RS[NV+i]  =  RS[NM*NV+i];
          for( i = 1; i <= NV+na; i++ )
            RS[i]  =  -RS[i];
          NM  =  1;
        }
      }
      if(  c_EES == EES_D  ){
        free_2d( p_aa );
        free_2d( aa   );
      }
      free_2d( keep_EEx );
    }
    if( TP->CC  !=  0 )  break;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    iteration ++;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    if( TP->PF  ==  1 ){
      if( iteration == 1  ||  TP->PN < CC_Norm )
        M_Delete( DEL_ROW_A, DEL_COL_A, MAX_A, RS, a );
      else
        IP_Delete( DEL_ROW_A, MAX_A, IP );
      RS_Delete( DEL_ROW_A, MAX_A, RS );
      na--;
    }
    if( iteration == 1  ||  TP->PN < CC_Norm ){
      if( np  ==  0 ){
        PartialDiff_Function_FU_AUTO( *TP, par, x, F, PDF );
        if( nk == 1 )  PartialDiff_Function_FP_AUTO( *TP, par, x, F, PDF );
        PartialDiff_Function( *TP, par, x, PDF );
          if     (   c_EES == EES_D                           )   M_S    ( *TP, par,          x, PDF, a );
          else if(   c_EES == EES_S_E  ||  c_EES == EES_S_R   )   M_S_SFM( *TP, par,          x, PDF, a );
        if( nv == 1 )
          if     (   c_EES == EES_D                           )   M_V    ( *TP, par, BC, p_x, x     , a );
          else if(   c_EES == EES_S_E  ||  c_EES == EES_S_R   )   M_V_SFM( *TP, par,     p_x, x     , a );
        if( nk == 1 )
          if     (   c_EES == EES_D                           )   M_K    ( *TP, par, BC, d_x, x, PDF, a );
          else if(   c_EES == EES_S_E  ||  c_EES == EES_S_R   )   M_K_SFM( *TP, par,     d_x, x, PDF, a );
      }
      LU_Decomposition_F_PIVOT_MPI( TP->PROCS, TP->RANK, NM*NV+na, IP, a );
    }
    for( i = 1; i <= NM*NV+na; i++ )
      tRS[i]  =  RS[i];
    Calculate_Solution_F_PIVOT( NM*NV+na, IP, a, RS );
    if( TP->PF == 1 ){   na ++;  IP_Addition( DEL_ROW_A, MAX_A, IP );   }
    if( TP->nv2  ==  1 )
      RS[IP[NM*NV+np+nv]]  =  0.0;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    if( TP->RANK  ==  0 ){
      if( iteration == 1 )  Output_Sol_Iter( NM*NV+na, iteration, IP, tRS, RS, x, 'w' );
      else                  Output_Sol_Iter( NM*NV+na, iteration, IP, tRS, RS, x, 'a' );
    }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    if( c_CM  !=  1 )
      for( i = 1; i <= NM*NV+na; i++ )
        x[i]  -=  RS[IP[i]];
    else{
      NM  =  TP->NM;
      double *del_x;
      double **MM;
      del_x  =  malloc_1d( NV+na+1 );
      MM     =  malloc_2d( NV, NV  );
      for( i = 1; i <= NV+na; i++ )  del_x[  i]   =  RS[IP[i]];   del_x[NV+na+1] = 1.0;
      for( i = 1; i <= NV   ; i++ )  x[      i]  +=  del_x[   i];
      for( i = 1; i <= na   ; i++ )  x[NM*NV+i]  +=  del_x[NV+i];
      int j, i_NM;
      for( i_NM = 1; i_NM <= NM-1; i_NM++ ){
        for( i = 1; i <= NV; i++ )
          for( j = 1; j <= NV; j++ )
            MM[i][j]  =  M[(i_NM-1)*NV+i][j];
        MatrixProduct_21s( NV, NV, MM, del_x, del_x );
        for( j = 1; j <= NV; j++ )
          for( i = 1; i <= na+1; i++ )
            del_x[j]  +=  M[(i_NM-1)*NV+j][NV+i]*del_x[NV+i];
        for( i = 1; i <= NV; i++ )
          x[i_NM*NV+i]  +=  del_x[i];
      }
      free_1d( del_x );
      free_2d( MM    );
    }
    if( nk  ==  1 ){
      par[TP->CP]  =  x[NM*NV+na];
      Diffusion( *TP, par );
    }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  }
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  if( iteration != 0  &&  nk == 1 ){
/*
    if( TP->PN  <  CC_Norm ){
      if( np  ==  0 ){
        PartialDiff_Function_FU_AUTO( *TP, par, x, F, PDF );
        PartialDiff_Function_FP_AUTO( *TP, par, x, F, PDF );
        PartialDiff_Function( *TP, par, x, PDF );
        M_S( *TP, par, x, PDF, a );
        if( nv == 1 )  M_V( *TP, par, BC, p_x, x, a );
        M_K( *TP, par, BC, d_x, x, PDF, a );
      }else{
iteration=100;
        RS_P( iteration, TP, par, BC, d_x, p_x, x, RS, a, &CC_Norm );
}
      LU_Decomposition_F_MPI( TP->PROCS, TP->RANK, NV+na, a );
//    LU_Decomposition_F( NV+na, a );
    }
*/
    if( TP->CC  ==  1 ){
      if( c_CM  !=  1 ){
        if( TP->PF  ==  1 )
          for( i = DEL_ROW_A+1; i <= MAX_A; i++ )
            IP[i-1]  =  IP[i];
        RightSide_Of_Dot( NM*NV+na-npf, RS );
        Calculate_Solution_F_PIVOT( NM*NV+na-npf, IP, a, RS );
        if( TP->PF  ==  1 ){
          for( i = MAX_A-1; i >= DEL_ROW_A; i-- )
            IP[i+1]  =  IP[i];
          IP[DEL_ROW_A]  =  NM*NV+na;
        }
        if( NM  ==  1 ){
          for( i = 1; i <= NM*NV+na; i++ )   d_x[      i] = RS[IP[      i]];
        }else{
          for( i =    1; i <=    NV; i++ )   d_x[      i] = RS[IP[      i]];
          for( i = NV+1; i <= NM*NV; i++ )   d_x[      i] = 0.0;
          for( i =    1; i <= na   ; i++ )   d_x[NM*NV+i] = RS[IP[NM*NV+i]];
        }
        normalize_dot_x( NM*NV+na, d_x, &i );   if( i == 1 ) TP->CC = 9;
      }else{
        NM  =  TP->NM;
        double **MM;
        MM  =  malloc_2d( NV, NV );
        NM  =  1;
        for( i = 1; i <= NM*NV+na-1; i++ )
          RS[i]  =  0.0;
        RS[NM*NV+na]  =  1.0;
        if( TP->PF  ==  1 ){
          IP_Delete( DEL_ROW_A, MAX_A, IP );
          RS_Delete( DEL_ROW_A, MAX_A, RS );
          na--;
        }
/*        int i_NM;*/
/*        double *C;*/
/*        C   =  malloc_1d( NV     );*/
/*        for( i_NM = 1; i_NM <= NM; i_NM++ )*/
/*          for( i = 1; i <= NV; i++ )*/
/*            M[(i_NM-1)*NV+i][NV+na+1]  =  RS[(i_NM-1)*NV+i];*/
/*        for( i_NM = 2; i_NM <= NM; i_NM++ ){*/
/*          for( i = 1; i <= NV; i++ )*/
/*            for( j = 1; j <= NV; j++ )*/
/*              MM[i][j]  =  M[(i_NM-1)*NV+i][j];*/
/*          MatrixProduct_21( NV, NV, MM, RS, C );*/
/*          for( i = 1; i <= NV; i++ )*/
/*            RS[i]  =  C[i] + RS[(i_NM-1)*NV+i];*/
/*        }*/
/*        free_1d( C );*/
/*        for( i = 1; i <= na; i++ )*/
/*          RS[NV+i]  =  RS[NM*NV+i];*/
/*        for( i = 1; i <= NV+na; i++ )*/
/*          RS[i]  =  -RS[i];*/
        Calculate_Solution_F_PIVOT( NM*NV+na, IP, a, RS );
        if( TP->PF == 1 ){   na ++;  IP_Addition( DEL_ROW_A, MAX_A, IP );   }
/*        double tmp;*/
        double *del_x;
        del_x  =  malloc_1d( NV+na+1 );
        for( i = 1; i <= NV+na; i++ )  del_x[    i]  =  RS[IP[i]];   del_x[NV+na+1] = 1.0;
        for( i = 1; i <= NV   ; i++ )  d_x[      i]  =  del_x[   i];
        for( i = 1; i <= na   ; i++ )  d_x[NM*NV+i]  =  del_x[NV+i];
/*        for( i_NM = 1; i_NM <= NM-1; i_NM++ ){*/
/*          for( i = 1; i <= NV; i++ )*/
/*            for( j = 1; j <= NV; j++ )*/
/*              MM[i][j]  =  M[(i_NM-1)*NV+i][j];*/
/*          MatrixProduct_21s( NV, NV, MM, del_x, del_x );*/
/*          for( j = 1; j <= NV; j++ )*/
/*            for( i = 1; i <= na+1; i++ )*/
/*              del_x[j]  +=  M[(i_NM-1)*NV+j][NV+i]*del_x[NV+i];*/
/*          for( i = 1; i <= NV; i++ )*/
/*            d_x[i_NM*NV+i]  +=  del_x[i];*/
/*        }*/
        free_1d( del_x );
        free_2d( MM    );
        normalize_dot_x( NM*NV+na, d_x, &i );   if( i == 1 ) TP->CC = 9;
        NM  =  TP->NM;
        for( i = 1; i <= na; i++ ){
          d_x[NM*NV+i]  =  d_x[NV+i];
          d_x[   NV+i]  =  0.0;
        }
      }
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 1; i <= NM*NV; i++ ){
    t__d_x[i]  =  d_x[i];
    t__p_x[i]  =  p_x[i];
    t__x  [i]  =  x  [i];
  }
  if( np  ==  1 ){
    t__d_x[NM*NV+1]  =  d_x[NM*NV+np];
    t__p_x[NM*NV+1]  =  p_x[NM*NV+np];
    t__x  [NM*NV+1]  =  x  [NM*NV+np];
  }
  if( nv  ==  1 ){
    t__d_x[NM*NV+2]  =  d_x[NM*NV+np+nv];
    t__p_x[NM*NV+2]  =  p_x[NM*NV+np+nv];
    t__x  [NM*NV+2]  =  x  [NM*NV+np+nv];
  }
  if( nk  ==  1 ){
    t__d_x[NM*NV+3]  =  d_x[NM*NV+na];
    t__p_x[NM*NV+3]  =  p_x[NM*NV+na];
    t__x  [NM*NV+3]  =  x  [NM*NV+na];
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_1i( IP       );
  free_1d( d_x      );
  free_1d( p_x      );
  free_1d( x        );
  if( np  ==  0 )
  free_1d( F        );
  free_1d( RS       );
  free_1d( tRS      );
  if( np  ==  0 )
  free_2d( PDF      );
  if(   np == 1  &&  (TP->EES==EES_D_M || TP->EES==EES_S_E_M || TP->EES==EES_S_R_M)   )
  free_2d( keep_p_x );
  free_2d( a        );
  if( c_CM  ==  1 )
  free_2d( M        );
  FREE_1C( PRINTF   );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  Newton_Method_S_ILUCGS( TEMP *TP, int *c_G, double *par, double *BC, double *x )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int N_VAR, N, NV;
  N_VAR  =  TP->N_VAR;
  N      =  TP->N;
  NV     =  TP->NV;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j;
  int    iteration;
  int    N1, N2, M1=1, M2=1;
  int    IER;
  int    *IP;
  double CC_Norm;
  double *F;
  double **PDF;
  double *RS;
  double **a;
  double *sol_x;
  double *t_RS, *t_a, *t_sol_x;
  char   *PRINTF;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  IP       =  malloc_1i( NV          );
  F        =  malloc_1d( NV          );
  PDF      =  malloc_2d( N_VAR, NV+N );
  RS       =  malloc_1d( NV          );
  a        =  malloc_2d( NV   , NV   );
  sol_x    =  malloc_1d( NV          );
  t_RS     =  MALLOC_1D( NV          );
  t_a      =  MALLOC_1D( 7*NV        );
  t_sol_x  =  MALLOC_1D( NV          );
  PRINTF   =  MALLOC_1C( SIZE        );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  if     ( N_VAR  ==  1 ){  M1 = N-2;  M2 = N-1;  }
  else if( N_VAR  ==  2 ){  M1 = N-1;  M2 = N;    }
  else if( N_VAR  ==  3 ){  M1 = N;    M2 = 2*N;  }
  N1  =  NV + M2;
  N2  =     - M2;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  par[TP->CP]  +=  TP->ds;
  x[NV+3]       =  par[TP->CP];
  Diffusion( *TP, par );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  for( i = 1; i <= NV; i++ )
    IP[i]  =  i;
  for( i = 0; i <= NV-1; i++ )
    t_sol_x[i]  =  x[i+1];
/*-------------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------------*/
/*  a[i][1] is value of M2 remote element from the opposite angle in the left    */
/*  a[i][2]             M1                                               left    */
/*  a[i][3]              1                                               left    */
/*  a[i][4] is value of           element from the opposite angle                */
/*  a[i][5] is value of  1 remote element from the opposite angle in the right   */
/*  a[i][6]             M1                                               right   */
/*  a[i][7]             M2                                               right   */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*  1  <  M1  <  M2  <  NV                                                       */
/*  N1  >=  NV  +  M2                                                            */
/*  N2  <=      -  M2                                                            */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*  ( Model and kind ) of Argument                                               */
/*                                                                               */
/*  double  a[N2:NV+M2][7]                                                       */
/*  int     NV, N1, N2, M1, M2                                                   */
/*  double  RS[NV]                                                               */
/*  double  EPS                                                                  */
/*  int     ITR                                                                  */
/*  double  S                                                                    */
/*  double  sol_x[N2:NV+M2]                                                      */
/*  double  D[N2:NV+M2], P[N2:NV+M2], Q[N2:NV+M2], R[N2:NV+M2]                   */
/*  double  R0[NV]     , E[NV]      , H[NV]                                      */
/*  double  W[N2:NV+M2]                                                          */
/*  int     IER                                                                  */
/*-------------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------------*/
  iteration  =  0;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  while( 1 ){
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    if( TP->RANK  ==  0 )
      NW_Gnuplot( iteration, *TP, c_G, par, BC, x );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    Reaction( *TP, par, x, F );
    RS_S( *TP, par, BC, x, F, RS );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    NM_Norm( iteration, TP, par, RS, PRINTF, &CC_Norm );
    if( TP->RANK  ==  0 )
      NM_Stdout( iteration, *TP, CC_Norm, PRINTF, par, x ); 
    if( TP->CC  !=  0 )  break;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    iteration ++;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    PartialDiff_Function_FU_AUTO( *TP, par, x, F, PDF );
    PartialDiff_Function( *TP, par, x, PDF );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    M_S( *TP, par, x, PDF, a );
    M_S_ILUCGS( M1, M2, NV, a );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    for( i = 0; i <= NV-1; i++ ){
      t_RS[i]  =  RS[i+1];
      for( j = 0; j <= 6; j++ )
        t_a[7*i+j]  =  a[i+1][j+1];
    }
    ilucgs1_( &NV, &N1, &N2, &M1, &M2, t_RS, t_a, t_sol_x, &IER );
    if( IER  !=  0 ){
      sprintf( PRINTF, "ILUCGS IER Err !!(%d)\n", IER );
      END_PROGRAM( PRINTF, 10, 0 );
      TP->CC  =  -1;
      break;
    }
    for( i = 0; i <= NV-1; i++ ){
      sol_x[i+1]  =  t_sol_x[i];
      t_sol_x[i]  =  0.0;
    }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    if( TP->RANK  ==  0 ){
      if( iteration == 1 )  Output_Sol_Iter( NV, iteration, IP, RS, sol_x, x, 'w' );
      else                  Output_Sol_Iter( NV, iteration, IP, RS, sol_x, x, 'a' );
    }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    for( i = 1; i <= NV; i++ )
      x[i]  -=  sol_x[i];
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  }
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  free_1i( IP      );
  free_1d( F       );
  free_2d( PDF     );
  free_1d( RS      );
  free_2d( a       );
  free_1d( sol_x   );
  FREE_1D( t_RS    );
  FREE_1D( t_a     );
  FREE_1D( t_sol_x );
  FREE_1C( PRINTF  );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/
