/*-------------------------------------------------------------------------------*/
#include <stdio.h>
#include <mpi.h>
#include "StructType_Definition.h"
#include "function_declartion.h"
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct( int L, int M, double **A, int N, double **B, double **C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k;
  double temp;
/*-------------------------------------------------------------------------------*/
   
/*-------------------------------------------------------------------------------*/
  for( i = 1; i <= L; i++ ){
    for( j = 1; j <= N; j++ ){
      temp  =  0.0;
      for( k = 1; k <= M; k++ )
        temp  +=  A[i][k] * B[k][j];
      C[i][j]  =  temp;
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct_MPI( int PROCS, int RANK, int L, int M, double **A, int N, double **B, double **C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k;
  double temp;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    is, ie;
  int    *len, *disp;
  double **t_C;
  len   =  malloc_1i( PROCS+1 );
  disp  =  malloc_1i( PROCS+1 );
  Partition_1dim( PROCS, RANK, L, N, &is, &ie, len, disp );
  t_C   =  malloc_2d( ie-is+1, L );
/*-------------------------------------------------------------------------------*/
   
/*-------------------------------------------------------------------------------*/
  for( i = is; i <= ie; i++ ){
    for( j = 1; j <= N; j++ ){
      temp  =  0.0;
      for( k = 1; k <= M; k++ )
        temp  +=  A[i][k] * B[k][j];
      t_C[i-is+1][j]  =  temp;
    }
  }
  MPI_Allgatherv( &t_C[1][1], len[RANK+1], MPI_DOUBLE, &C[1][1], &len[1], &disp[1], MPI_DOUBLE, MPI_COMM_WORLD );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_1i( len  );
  free_1i( disp );
  free_2d( t_C  );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  MatrixProduct_MPI1( int PROCS, int RANK, int is    , int ie, int *len  , int *disp , double **t_C, \
                          int L    , int M   , double **A, int N , double **B, double **C )
/*-------------------------------------------------------------------------------*/
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k;
  double temp;
/*-------------------------------------------------------------------------------*/
   
/*-------------------------------------------------------------------------------*/
  for( i = is; i <= ie; i++ ){
    for( j = 1; j <= N; j++ ){
      temp  =  0.0;
      for( k = 1; k <= M; k++ )
        temp  +=  A[i][k] * B[k][j];
      t_C[i-is+1][j]  =  temp;
    }
  }
  MPI_Allgatherv( &t_C[1][1], len[RANK+1], MPI_DOUBLE, &C[1][1], &len[1], &disp[1], MPI_DOUBLE, MPI_COMM_WORLD );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/
