/*-------------------------------------------------------------------------------*/
#include <stdio.h>
#include <math.h>
#include <mpi.h>
#include "StructType_Definition.h"
#include "function_declartion.h"
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  LU_Decomposition_F_PIVOT_MPI( int PROCS, int RANK, int N, int *IP, double **a )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, j, k, l, t_IP;
  int    *MAP;
  double *fa, *MAX_a;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  MAP    =  malloc_1i( N       );
  fa     =  malloc_1d( 2       );
  MAX_a  =  malloc_1d( 2*PROCS );
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  for( i = 1; i <= N; i++ ){
    MAP[i]  =  (i-1) % PROCS;
    IP[i]   =  i;
  }
  for( k = 1; k <= N-1; k++ ){
    fa[1]  =  0.0;
    l      =  k;
    for( i = k; i <= N; i++ ){
      if( RANK  ==  MAP[IP[i]] ){
        if( fa[1]  <  fabs(a[IP[i]][k]) ){
          fa[1]  =  fabs(a[IP[i]][k]);
          fa[2]  =  (double)i;
        }
      }
    }
    MPI_Gather( &fa[1], 2, MPI_DOUBLE, &MAX_a[1], 2, MPI_DOUBLE, 0, MPI_COMM_WORLD );
    if( RANK  ==  0 ){
      l  =  (int)fa[2];
      for( i = 3; i <= 2*PROCS; i+=2 ){
        if( fa[1]  <  fabs(MAX_a[i]) ){
          fa[1]  =  fabs(MAX_a[i]);
          l      =  (int)MAX_a[i+1];
        }
      }
    }
    MPI_Bcast( &l, 1, MPI_INT, 0, MPI_COMM_WORLD );
    if( k  !=  l ){
      t_IP   =  IP[k];
      IP[k]  =  IP[l];
      IP[l]  =  t_IP;
    }
    if( MAP[IP[k]]  ==  RANK )
      for( j = k+1; j <= N; j++ )
        a[IP[k]][j]  /=  a[IP[k]][k];
    MPI_Bcast( &a[IP[k]][k+1], N-k, MPI_DOUBLE, MAP[IP[k]], MPI_COMM_WORLD );
    for( i = k+1; i <= N; i++ )
      if( MAP[IP[i]]  ==  RANK )
        for( j = k+1; j <= N; j++ )
          a[IP[i]][j]  -=  a[IP[i]][k] * a[IP[k]][j];
  }
  for( i = 1; i <= N; i++ )
    MPI_Bcast( &a[IP[i]][1], N, MPI_DOUBLE, MAP[IP[i]], MPI_COMM_WORLD );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_1i( MAP   );
  free_1d( fa    );
  free_1d( MAX_a );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  Calculate_Solution_F_PIVOT( int N, int *IP, double **a, double *RS )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int  i, j, k;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  for( k = 1; k <= N-1; k++ ){
    RS[IP[k]]  /=  a[IP[k]][k];
    for( i = k+1; i <= N; i++ )
      RS[IP[i]]  -=  a[IP[i]][k] * RS[IP[k]];
  }
  RS[IP[N]]  /=  a[IP[N]][N];
  for( k = N-1; k >= 1; k-- )
    for( j = k+1; j <= N; j++ )
      RS[IP[k]]  -=  a[IP[k]][j] * RS[IP[j]];
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/



/*********************************************************************************/
/*********************************************************************************/
/*********************************************************************************/
/*********************************************************************************/




/*-------------------------------------------------------------------------------*/
void  LU_Decomposition_F_MPI( int PROCS, int RANK, int N, double **a )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int i, j, k;
  int *MAP;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  MAP  =  malloc_1i( N );
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  for( i = 1; i <= N; i++ )
    MAP[i]  =  (i-1) % PROCS;
  for( k = 1; k <= N-1; k++ ){
    if( MAP[k]  ==  RANK ){
      for( j = k+1; j <= N; j++ )
        a[k][j]  /=  a[k][k];
    }
    MPI_Bcast( &a[k][k+1], N-k, MPI_DOUBLE, MAP[k], MPI_COMM_WORLD );
    for( i = k+1; i <= N; i++ )
      if( MAP[i]  ==  RANK )
        for( j = k+1; j <= N; j++ )
          a[i][j]  -=  a[i][k] * a[k][j];
  }
  for( i = 1; i <= N; i++ )
    MPI_Bcast( &a[i][1], N, MPI_DOUBLE, MAP[i], MPI_COMM_WORLD );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_1i( MAP );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/



/*********************************************************************************/
/*********************************************************************************/
/*********************************************************************************/
/*********************************************************************************/



/*-------------------------------------------------------------------------------*/
void  LU_Decomposition_F( int N, double **a )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int i, j, k;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  for( k = 1; k <= N-1; k++ )
    for( j = k+1; j <= N; j++ )
      a[k][j]  /=  a[k][k];
  for( i = k+1; i <= N; i++ )
    for( j = k+1; j <= N; j++ )
      a[i][j]  -=  a[i][k] * a[k][j];
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  Calculate_Solution_F( int N, double **a, double *RS )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int i, j, k;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  for( k = 1; k <= N-1; k++ ){
    RS[k]  /=  a[k][k];
    for( i = k+1; i <= N; i++ )
      RS[i]  -=  a[i][k] * RS[k];
  }
  RS[N]  /=  a[N][N];
  for( k = N-1; k >= 1; k-- )
    for( j = k+1; j <= N; j++ )
      RS[k]  -=  a[k][j] * RS[j];
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/



/*********************************************************************************/
/*********************************************************************************/
/*********************************************************************************/
/*********************************************************************************/



/*-------------------------------------------------------------------------------*/
void  LU_Decomposition_EE( TEMP TP, double **a )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, Vi, i_VAR;
  double sum;
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  if( TP.BCS == 1  ||  TP.BCS == 2  ||  TP.BCS == 4 ){
    for( i_VAR = 1; i_VAR <= TP.N_VAR; i_VAR++ ){
      Vi  =  (i_VAR-1)*TP.N;
      for( i = 1+Vi; i <= TP.N-1+Vi; i++ ){
        a[3][i  ]  =  a[3][i] / a[1][i];
        a[1][i+1]  =  a[1][i+1] - a[3][i] * a[2][i+1];
      }
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  else if( TP.BCS  ==  3 ){
    for( i_VAR = 1; i_VAR <= TP.N_VAR; i_VAR++ ){
      Vi  =  (i_VAR-1)*TP.N;
      a[3][1+Vi]  =  a[3][1+Vi] / a[1][1+Vi];
      a[5][1+Vi]  =  a[5][1+Vi] / a[1][1+Vi];
      for( i = 2+Vi; i <= TP.N-2+Vi; i++ ){
        a[1][i]  =  a[1][i] - a[3][i-1]*a[2][i];
        a[3][i]  =  a[3][i] / a[1][i];
        a[5][i]  =  - a[2][i] * a[5][i-1] / a[1][i];
      }
      i        =  TP.N-1+Vi;
      a[1][i]  =  a[1][i] - a[3][i-1]*a[2][i];
      a[5][i]  =  ( a[3][i]-a[2][i]*a[5][i-1] ) / a[1][i];
      for( i = 2+Vi; i <= TP.N-2+Vi; i++ )
        a[4][i]  =  - a[3][i-1] * a[4][i-1];
      i = TP.N-1+Vi;
      a[4][i] = a[2][TP.N+Vi] - a[3][i-1] * a[4][i-1];   
      sum  =  0.0;
      for( i = 1+Vi; i <= TP.N-1+Vi; i++ )
        sum  +=  a[4][i] * a[5][i];
      a[4][TP.N+Vi]  =  a[1][TP.N+Vi] - sum;
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  Calculate_Solution_EE( TEMP TP, double *RS, double **a, double *x )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i, Vi, i_VAR;
  double sum;
  double *z;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  z  =  malloc_1d( TP.N );
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  if( TP.BCS == 1  ||  TP.BCS == 2   ||  TP.BCS == 4 ){
    for( i_VAR = 1; i_VAR <= TP.N_VAR; i_VAR++ ){
      Vi  =  (i_VAR-1)*TP.N;
      z[1]  =  RS[1+Vi] / a[1][1+Vi];
      for( i = 2+Vi; i <= TP.N+Vi; i++ )
        z[i-Vi]  =  ( RS[i] - a[2][i] * z[i-1-Vi] ) / a[1][i];
      x[TP.N+Vi]  =  z[TP.N];
      for( i = TP.N-1+Vi; i >= 1+Vi; i-- )
        x[i]  =  z[i-Vi] - a[3][i] * x[i+1];
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  else if( TP.BCS  ==  3 ){
    for( i_VAR = 1; i_VAR <= TP.N_VAR; i_VAR++ ){
      Vi  =  (i_VAR-1)*TP.N;
      z[1]  =  RS[1+Vi] / a[1][1+Vi];
      for( i = 2+Vi; i <= TP.N-1+Vi; i++ )
        z[i-Vi]  =  ( RS[i]-a[2][i]*z[i-1-Vi] ) / a[1][i];
      sum  =  0.0;
      for( i = 1+Vi; i <= TP.N-1+Vi; i++ )
        sum  +=  a[4][i]*z[i-Vi];
      i        =  TP.N + Vi;
      z[i-Vi]  =  ( RS[i]-sum ) / a[4][i];
      x[i   ]  =  z[i-Vi];
      x[i-1 ]  =  z[i-1-Vi]-a[5][i-1]*x[i];
      for( i = TP.N-2+Vi; i >= 1+Vi; i-- )
        x[i]  =  z[i-Vi] - a[3][i] * x[i+1] - a[5][i] * x[TP.N+Vi];
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_1d( z );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/
