/*-------------------------------------------------------------------------------*/
#include <stdio.h>
#include "function_declartion.h"
#include "constant.h"
/*-------------------------------------------------------------------------------*/

#define THETA 0.5




/*-------------------------------------------------------------------------------*/
void  M_1n1v3dDNC( TEMP TP, double *par, double *BC, double **a )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    N;
  double theta;
  N      =  TP.N;
  theta  =  (double)THETA;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i;
  int    i_VAR;
  double T, VEL;
  double Rd, Rv;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  T   =  TP.T;
  VEL =  TP.VEL;
/*-------------------------------------------------------------------------------*/
/*                                                                               */
/*   a[1][1] a[3][1]                                       a[5][1  ]             */
/*   a[2][2] a[1][2] a[3][2]                                                     */
/*           a[2][3] a[1][3] a[3][3]                                             */
/*                                                                               */
/*                                                                               */
/*                           a[2][N-2] a[1][N-2] a[3][N-2]                       */
/*                                     a[2][N-1] a[1][N-1] a[3][N-1]             */
/*   a[4][1]                                     a[2][N  ] a[1][N  ]             */
/*                                                                               */
/*-------------------------------------------------------------------------------*/
  Rv  =  (VEL*T*TP.dt) / (2.0*TP.dx);
  Rv  =  0.0;
  for( i_VAR = 1; i_VAR <= TP.N_VAR; i_VAR++ ){
    Rd  =  (par[TP.N_EP+i_VAR]*T*TP.dt*theta) / ( TP.dx*TP.dx );
    for( i = 1; i <= N; i++ ){
      a[1][i+(i_VAR-1)*N]  =  1.0 + 2.0 * Rd;              /*  Opposite  */
      a[2][i+(i_VAR-1)*N]  =        1.0 * ( - Rd + Rv );   /*  Left      */
      a[3][i+(i_VAR-1)*N]  =        1.0 * ( - Rd - Rv );   /*  Right     */
    }
    a[2][1+(i_VAR-1)*N]  =  0.0;
    a[3][N+(i_VAR-1)*N]  =  0.0;
    if( TP.BCS  ==  BCS_2 ){
      a[2][N+(i_VAR-1)*N]  +=  - Rd - Rv;
      a[3][1+(i_VAR-1)*N]  +=  - Rd + Rv;
    }else if( TP.BCS  ==  BCS_3 ){
      a[4][1+(i_VAR-1)*N]  =  - Rd - Rv;   /*  left  below  */
      a[5][1+(i_VAR-1)*N]  =  - Rd + Rv;   /*  upper right  */
    }else if( TP.BCS  ==  BCS_4 )
      a[2][N+(i_VAR-1)*N]  +=  - Rd - Rv;
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------------*/
void  RS_1n1v3dDNC( TEMP TP, double *par, double *BC, double *x, double *F, double *RS_EE )
{
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    N;
  double theta;
  N      =  TP.N;
  theta  =  1.0-(double)THETA;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  int    i;
  int    i_VAR, i_BC;
  double T, VEL;
  double Rd, Rv, Rv1, Rv2;
  double *BCX;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  BCX  =  malloc_1d( TP.N_VAR*2 );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  T    =  TP.T;
  VEL  =  TP.VEL;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  Generate_BC_Space( TP, BC, x, BCX );
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
  Rv   =  (VEL*T*TP.dt*theta) / (2.0*TP.dx);
  Rv   =  0.0;
  Rv1  =   (T*VEL*TP.dt) / (12.0*TP.dx);
  Rv2  =  -(T*VEL*TP.dt) / (12.0*TP.dx);
  i_BC  =  1;
  for( i_VAR = 1; i_VAR <= TP.N_VAR; i_VAR++ ){
    if( TP.BCS  ==  BCS_1 ){
      i  =  1;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*BCX[i_BC]+BCX[i_BC]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*BCX[i_BC]+BCX[i_BC]);
      i  =  2;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1]+BCX[i_BC]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+BCX[i_BC]);
      i_BC ++;
      for( i = 3; i <= TP.N-2; i++ )
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]);
      i  =  TP.N-1;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-BCX[i_BC]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]) \
                              +  Rv2*( BCX[i_BC]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]);
      i  =  TP.N;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-BCX[i_BC]+8.0*BCX[i_BC]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]) \
                              +  Rv2*( BCX[i_BC]-4.0*BCX[i_BC]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]);
      i_BC ++;
    }else if( TP.BCS  ==  BCS_2 ){
      i  =  1;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1+2]+x[(i_VAR-1)*N+i-2+4]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1+2]+x[(i_VAR-1)*N+i-2+4]);
      i  =  2;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2+2]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2+2]);
      for( i = 3; i <= TP.N-2; i++ )
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]);
      i  =  TP.N-1;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2-2]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2-2]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]);
      i  =  TP.N;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2-4]+8.0*x[(i_VAR-1)*N+i+1-2]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2-4]-4.0*x[(i_VAR-1)*N+i+1-2]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]);
    }else if( TP.BCS  ==  BCS_3 ){
      i  =  1;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1+N]+x[(i_VAR-1)*N+i-2+N]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1+N]+x[(i_VAR-1)*N+i-2+N]);
      i  =  2;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2+N]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2+N]);
      for( i = 3; i <= TP.N-2; i++ )
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]);
      i  =  TP.N-1;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2-N]+8.0*x[(i_VAR-1)*N+i+1]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2-N]-4.0*x[(i_VAR-1)*N+i+1]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]);
      i  =  TP.N;
        RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i] \
                              +  Rv1*(-x[(i_VAR-1)*N+i+2-N]+8.0*x[(i_VAR-1)*N+i+1-N]                     -8.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]) \
                              +  Rv2*( x[(i_VAR-1)*N+i+2-N]-4.0*x[(i_VAR-1)*N+i+1-N]+6.0*x[(i_VAR-1)*N+i]-4.0*x[(i_VAR-1)*N+i-1]+x[(i_VAR-1)*N+i-2]);
    }
/*    for( i = 1; i <= TP.N; i++ )*/
/*      RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i];*/
/*    Rv   =  (VEL*T*TP.dt) / (2.0*TP.dx);*/
/*      i  =  1;*/
/*        RS_EE[(i_VAR-1)*N+i]  +=  Rv*(x[(i_VAR-1)*N+i+1]-x[(i_VAR-1)*N+i-1+N]);*/
/*      for( i = 2; i <= TP.N-1; i++ )*/
/*        RS_EE[(i_VAR-1)*N+i]  +=  Rv*(x[(i_VAR-1)*N+i+1]-x[(i_VAR-1)*N+i-1]);*/
/*      i  =  TP.N;*/
/*        RS_EE[(i_VAR-1)*N+i]  +=  Rv*(x[(i_VAR-1)*N+i+1-N]-x[(i_VAR-1)*N+i-1]);*/
/*    for( i = 1; i <= TP.N; i++ )*/
/*      RS_EE[(i_VAR-1)*N+i]  =  x[(i_VAR-1)*N+i] + T*TP.dt*F[(i_VAR-1)*N+i];*/
/*************/
/*Rv1=1.0;*/
/*Rv2=-Rv1/8.0;*/
/*    Rv   =  (VEL*T*TP.dt) / ((2.0*Rv1+4.0*Rv2)*TP.dx);*/
/*      i  =  1;*/
/*        RS_EE[(i_VAR-1)*N+i]  +=  Rv*(Rv2*x[(i_VAR-1)*N+i+2]+Rv1*x[(i_VAR-1)*N+i+1]-Rv1*x[(i_VAR-1)*N+i-1+N]-Rv2*x[(i_VAR-1)*N+i-2+N]);*/
/*      i  =  2;*/
/*        RS_EE[(i_VAR-1)*N+i]  +=  Rv*(Rv2*x[(i_VAR-1)*N+i+2]+Rv1*x[(i_VAR-1)*N+i+1]-Rv1*x[(i_VAR-1)*N+i-1]-Rv2*x[(i_VAR-1)*N+i-2+N]);*/
/*      for( i = 3; i <= TP.N-2; i++ )*/
/*        RS_EE[(i_VAR-1)*N+i]  +=  Rv*(Rv2*x[(i_VAR-1)*N+i+2]+Rv1*x[(i_VAR-1)*N+i+1]-Rv1*x[(i_VAR-1)*N+i-1]-Rv2*x[(i_VAR-1)*N+i-2]);*/
/*      i  =  TP.N-1;*/
/*        RS_EE[(i_VAR-1)*N+i]  +=  Rv*(Rv2*x[(i_VAR-1)*N+i+2-N]+Rv1*x[(i_VAR-1)*N+i+1]-Rv1*x[(i_VAR-1)*N+i-1]-Rv2*x[(i_VAR-1)*N+i-2]);*/
/*      i  =  TP.N;*/
/*        RS_EE[(i_VAR-1)*N+i]  +=  Rv*(Rv2*x[(i_VAR-1)*N+i+2-N]+Rv1*x[(i_VAR-1)*N+i+1-N]-Rv1*x[(i_VAR-1)*N+i-1]-Rv2*x[(i_VAR-1)*N+i-2]);*/
/*    Rd  =  (par[TP.N_EP+i_VAR]*T*TP.dt*theta) / ( TP.dx*TP.dx );*/
/*    i  =  1;*/
/*      RS_EE[(i_VAR-1)*N+i]  +=  Rd*(x[(i_VAR-1)*N+i+1  ]-2.0*x[(i_VAR-1)*N+i]+x[(i_VAR-1)*N+i-1+N]);*/
/*    for( i = 2; i <= N-1; i++ )*/
/*      RS_EE[(i_VAR-1)*N+i]  +=  Rd*(x[(i_VAR-1)*N+i+1  ]-2.0*x[(i_VAR-1)*N+i]+x[(i_VAR-1)*N+i-1  ]);*/
/*    i  =  N;*/
/*      RS_EE[(i_VAR-1)*N+i]  +=  Rd*(x[(i_VAR-1)*N+i+1-N]-2.0*x[(i_VAR-1)*N+i]+x[(i_VAR-1)*N+i-1  ]);*/
  }
  i_BC  =  1;
  if( TP.BCS  ==  BCS_1 ){
    for( i = 1; i <= TP.N_VAR; i++ ){
      RS_EE[(i-1)*TP.N+1   ]  +=  (par[TP.N_EP+i]*BCX[i_BC]*TP.dt*T) / (TP.dx*TP.dx) - Rv*BCX[i_BC];   i_BC++;
      RS_EE[(i-1)*TP.N+TP.N]  +=  (par[TP.N_EP+i]*BCX[i_BC]*TP.dt*T) / (TP.dx*TP.dx) + Rv*BCX[i_BC];   i_BC++;
    }
  }else if( TP.BCS  ==  BCS_2 ){
    for( i = 1; i <= TP.N_VAR; i++ ){
      RS_EE[(i-1)*TP.N+1   ]  +=  ((par[TP.N_EP+i]*T*TP.dt)/(TP.dx*TP.dx)) * (-2.0*BC[i_BC]*TP.dx) + Rv*2.0*BC[i_BC]*TP.dx;   i_BC++;
      RS_EE[(i-1)*TP.N+TP.N]  +=  ((par[TP.N_EP+i]*T*TP.dt)/(TP.dx*TP.dx)) * ( 2.0*BC[i_BC]*TP.dx) - Rv*2.0*BC[i_BC]*TP.dx;   i_BC++;
    }
  }else if( TP.BCS  ==  BCS_4 ){
    for( i = 1; i <= TP.N_VAR; i++ ){
      RS_EE[(i-1)*TP.N+1   ]  +=  (par[TP.N_EP+i]*BCX[i_BC]*TP.dt*T) / (TP.dx*TP.dx) - Rv*BCX[i_BC];   i_BC++;
      RS_EE[(i-1)*TP.N+TP.N]  +=  ((par[TP.N_EP+i]*T*TP.dt)/(TP.dx*TP.dx)) * ( 2.0*BC[i_BC]*TP.dx) - Rv*2.0*BC[i_BC]*TP.dx;   i_BC++;
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  free_1d( BCX );
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
}
/*-------------------------------------------------------------------------------*/
