/*-------------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
#define  alloc_error_check(p){ \
  if( (p)  ==  NULL ){ \
    fprintf(stderr, "Allocation Failure!\n"); \
    exit(1); \
  } \
}
/*-------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------*/
int  *alloc_1d_int(int n1)
{
  int *i;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  i = (int *) malloc(sizeof(int) * n1);
  alloc_error_check(i);
  return i;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  free_1d_int(int *i)
{
  free(i);
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
double  *alloc_1d_double(int n1)
{
  double *d;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  d = (double *) malloc(sizeof(double) * n1);
  alloc_error_check(d);
  return d;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  free_1d_double(double *d)
{
  free(d);
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
char  *alloc_1d_char(int n1)
{
  char *c;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  c = (char *) malloc(sizeof(char) * n1);
  alloc_error_check(c);
  return c;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  free_1d_char(char *c)
{
  free(c);
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
int  **alloc_2d_int(int n1, int n2)
{
  int **ii, *i;
  int j;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  ii = (int **) malloc(sizeof(int *) * n1);
  alloc_error_check(ii);
  i = (int *) malloc(sizeof(int) * n1 * n2);
  alloc_error_check(i);
  ii[0] = i;
  for (j = 1; j < n1; j++) {
    ii[j] = ii[j - 1] + n2;
  }
  return ii;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  free_2d_int(int **ii)
{
  free(ii[0]);
  free(ii);
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
double  **alloc_2d_double(int n1, int n2)
{
  double **dd, *d;
  int j;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  dd = (double **) malloc(sizeof(double *) * n1);
  alloc_error_check(dd);
  d = (double *) malloc(sizeof(double) * n1 * n2);
  alloc_error_check(d);
  dd[0] = d;
  for (j = 1; j < n1; j++) {
    dd[j] = dd[j - 1] + n2;
  }
  return dd;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  free_2d_double(double **dd)
{
  free(dd[0]);
  free(dd);
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
char  **alloc_2d_char(int n1, int n2)
{
  char **cc, *c;
  int j;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  cc = (char **) malloc(sizeof(char *) * n1);
  alloc_error_check(cc);
  c = (char *) malloc(sizeof(char) * n1 * n2);
  alloc_error_check(c);
  cc[0] = c;
  for (j = 1; j < n1; j++) {
    cc[j] = cc[j - 1] + n2;
  }
  return cc;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  free_2d_char(char **cc)
{
  free(cc[0]);
  free(cc);
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
int  ***alloc_3d_int(int n1, int n2, int n3)
{
  int ***iii, **ii, *i;
  int j;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  iii = (int ***) malloc(sizeof(int **) * n1);
  alloc_error_check(iii);
  ii = (int **) malloc(sizeof(int *) * n1 * n2);
  alloc_error_check(ii);
  iii[0] = ii;
  for (j = 1; j < n1; j++) {
    iii[j] = iii[j - 1] + n2;
  }
  i = (int *) malloc(sizeof(int) * n1 * n2 * n3);
  alloc_error_check(i);
  ii[0] = i;
  for (j = 1; j < n1 * n2; j++) {
    ii[j] = ii[j - 1] + n3;
  }
  return iii;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  free_3d_int(int ***iii)
{
  free(iii[0][0]);
  free(iii[0]);
  free(iii);
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
double  ***alloc_3d_double(int n1, int n2, int n3)
{
  double ***ddd, **dd, *d;
  int j;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  ddd = (double ***) malloc(sizeof(double **) * n1);
  alloc_error_check(ddd);
  dd = (double **) malloc(sizeof(double *) * n1 * n2);
  alloc_error_check(dd);
  ddd[0] = dd;
  for (j = 1; j < n1; j++) {
    ddd[j] = ddd[j - 1] + n2;
  }
  d = (double *) malloc(sizeof(double) * n1 * n2 * n3);
  alloc_error_check(d);
  dd[0] = d;
  for (j = 1; j < n1 * n2; j++) {
    dd[j] = dd[j - 1] + n3;
  }
  return ddd;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  free_3d_double(double ***ddd)
{
  free(ddd[0][0]);
  free(ddd[0]);
  free(ddd);
}
/*-------------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------------*/
char  ***alloc_3d_char(int n1, int n2, int n3)
{
  char ***ccc, **cc, *c;
  int j;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  ccc = (char ***) malloc(sizeof(char **) * n1);
  alloc_error_check(ccc);
  cc = (char **) malloc(sizeof(char *) * n1 * n2);
  alloc_error_check(cc);
  ccc[0] = cc;
  for (j = 1; j < n1; j++) {
    ccc[j] = ccc[j - 1] + n2;
  }
  c = (char *) malloc(sizeof(char) * n1 * n2 * n3);
  alloc_error_check(c);
  cc[0] = c;
  for (j = 1; j < n1 * n2; j++) {
    cc[j] = cc[j - 1] + n3;
  }
  return ccc;
}
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
void  free_3d_char(char ***ccc)
{
  free(ccc[0][0]);
  free(ccc[0]);
  free(ccc);
}
/*-------------------------------------------------------------------------------*/
